# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import service
from google.protobuf import service_reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)



DESCRIPTOR = descriptor.FileDescriptor(
  name='recall/proto/rpc_meta.proto',
  package='rpc',
  serialized_pb='\n\x1brecall/proto/rpc_meta.proto\x12\x03rpc\"k\n\x08MetaInfo\x12\x0f\n\x07\x66low_id\x18\x01 \x02(\x05\x12\x14\n\x0cservice_name\x18\x02 \x02(\t\x12\x13\n\x0bmethod_name\x18\x03 \x02(\t\x12\x10\n\x08msg_name\x18\x04 \x02(\t\x12\x11\n\thas_error\x18\x1e \x01(\x08\"B\n\rErrorResponse\x12 \n\x08\x65rr_code\x18\x01 \x02(\x0e\x32\x0e.rpc.ErrorCode\x12\x0f\n\x07\x65rr_msg\x18\x02 \x02(\t\"%\n\x10HeartBeatRequest\x12\x11\n\tmagic_num\x18\x01 \x02(\x05\"(\n\x11HeartBeatResponse\x12\x13\n\x0breturn_code\x18\x01 \x02(\x05*N\n\tErrorCode\x12\x0b\n\x07SUCCESS\x10\x00\x12\x18\n\x14SERVER_SERVICE_ERROR\x10\x01\x12\x1a\n\x16SERVER_SERVICE_TIMEOUT\x10\x02\x32L\n\x0e\x42uiltinService\x12:\n\tHeartBeat\x12\x15.rpc.HeartBeatRequest\x1a\x16.rpc.HeartBeatResponseB\x03\x90\x01\x01')

_ERRORCODE = descriptor.EnumDescriptor(
  name='ErrorCode',
  full_name='rpc.ErrorCode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='SERVER_SERVICE_ERROR', index=1, number=1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='SERVER_SERVICE_TIMEOUT', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=294,
  serialized_end=372,
)


SUCCESS = 0
SERVER_SERVICE_ERROR = 1
SERVER_SERVICE_TIMEOUT = 2



_METAINFO = descriptor.Descriptor(
  name='MetaInfo',
  full_name='rpc.MetaInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='flow_id', full_name='rpc.MetaInfo.flow_id', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='service_name', full_name='rpc.MetaInfo.service_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='method_name', full_name='rpc.MetaInfo.method_name', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='msg_name', full_name='rpc.MetaInfo.msg_name', index=3,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='has_error', full_name='rpc.MetaInfo.has_error', index=4,
      number=30, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=36,
  serialized_end=143,
)


_ERRORRESPONSE = descriptor.Descriptor(
  name='ErrorResponse',
  full_name='rpc.ErrorResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='err_code', full_name='rpc.ErrorResponse.err_code', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='err_msg', full_name='rpc.ErrorResponse.err_msg', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=145,
  serialized_end=211,
)


_HEARTBEATREQUEST = descriptor.Descriptor(
  name='HeartBeatRequest',
  full_name='rpc.HeartBeatRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='magic_num', full_name='rpc.HeartBeatRequest.magic_num', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=213,
  serialized_end=250,
)


_HEARTBEATRESPONSE = descriptor.Descriptor(
  name='HeartBeatResponse',
  full_name='rpc.HeartBeatResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='return_code', full_name='rpc.HeartBeatResponse.return_code', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=252,
  serialized_end=292,
)

_ERRORRESPONSE.fields_by_name['err_code'].enum_type = _ERRORCODE
DESCRIPTOR.message_types_by_name['MetaInfo'] = _METAINFO
DESCRIPTOR.message_types_by_name['ErrorResponse'] = _ERRORRESPONSE
DESCRIPTOR.message_types_by_name['HeartBeatRequest'] = _HEARTBEATREQUEST
DESCRIPTOR.message_types_by_name['HeartBeatResponse'] = _HEARTBEATRESPONSE

class MetaInfo(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _METAINFO
  
  # @@protoc_insertion_point(class_scope:rpc.MetaInfo)

class ErrorResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _ERRORRESPONSE
  
  # @@protoc_insertion_point(class_scope:rpc.ErrorResponse)

class HeartBeatRequest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _HEARTBEATREQUEST
  
  # @@protoc_insertion_point(class_scope:rpc.HeartBeatRequest)

class HeartBeatResponse(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _HEARTBEATRESPONSE
  
  # @@protoc_insertion_point(class_scope:rpc.HeartBeatResponse)


_BUILTINSERVICE = descriptor.ServiceDescriptor(
  name='BuiltinService',
  full_name='rpc.BuiltinService',
  file=DESCRIPTOR,
  index=0,
  options=None,
  serialized_start=374,
  serialized_end=450,
  methods=[
  descriptor.MethodDescriptor(
    name='HeartBeat',
    full_name='rpc.BuiltinService.HeartBeat',
    index=0,
    containing_service=None,
    input_type=_HEARTBEATREQUEST,
    output_type=_HEARTBEATRESPONSE,
    options=None,
  ),
])

class BuiltinService(service.Service):
  __metaclass__ = service_reflection.GeneratedServiceType
  DESCRIPTOR = _BUILTINSERVICE
class BuiltinService_Stub(BuiltinService):
  __metaclass__ = service_reflection.GeneratedServiceStubType
  DESCRIPTOR = _BUILTINSERVICE

# @@protoc_insertion_point(module_scope)
